@@ -1,7 +1,7 @@
 # -*- coding: utf-8 -*-
 
 """
-Copyright (C) 2009,2010 Wolfgang Rohdewald <wolfgang@rohdewald.de>
+Copyright (C) 2009-2011 Wolfgang Rohdewald <wolfgang@rohdewald.de>
 
 kajongg is free software you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
@@ -15,7 +15,7 @@ GNU General Public License for more details.
 
 You should have received a copy of the GNU General Public License
 along with this program if not, write to the Free Software
-Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
+Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
 """
 
 from itertools import chain
@@ -37,7 +37,8 @@ class ClientTable(object):
     # pylint: disable=R0902
     # pylint: disable=R0913
     # pylint says too many args, too many instance variables
-    def __init__(self, tableid, gameid,  status, rulesetStr, playOpen, autoPlay, seed, playerNames, playersOnline):
+    def __init__(self, tableid, gameid, status, rulesetStr, playOpen, autoPlay, seed, playerNames,
+                 playersOnline, endValues):
         self.tableid = tableid
         self.gameid = gameid
         self.status = status
@@ -49,17 +50,17 @@ class ClientTable(object):
         self.seed = seed
         self.playerNames = playerNames
         self.playersOnline = playersOnline
-        self.myRuleset = None # if set, points to an identical local rulest
+        self.endValues = endValues
+        self.myRuleset = None # if set, points to an identical local ruleset
         allRulesets = Ruleset.availableRulesets() + PredefinedRuleset.rulesets()
-        selfHash = self.ruleset.hash
         for myRuleset in allRulesets:
-            if myRuleset.hash == selfHash:
+            if myRuleset == self.ruleset:
                 self.myRuleset = myRuleset
                 break
 
     def __str__(self):
         return 'Table %d %s rules %s players %s online %s' % (self.tableid or 0, self.status, self.ruleset.name,
-            ', '.join(self.playerNames),  ', '.join(str(x) for x in self.playersOnline))
+            ', '.join(self.playerNames), ', '.join(str(x) for x in self.playersOnline))
 
     def gameExistsLocally(self):
         """does the game exist in the data base of the client?"""
@@ -73,7 +74,7 @@ class ClientTable(object):
 class Client(pb.Referenceable):
     """interface to the server. This class only implements the logic,
     so we can also use it on the server for robot clients. Compare
-    with HumanClient(Client)"""
+    with Client1 and HumanClient(Client1)"""
 
     def __init__(self, username=None):
         """username is something like ROBOT 1 or None for the game server"""
@@ -130,9 +131,19 @@ class Client(pb.Referenceable):
             if not self.table:
                 raise Exception('client.readyForGameStart: tableid %d unknown' % tableid)
         if self.table.suspended:
-            self.game = RemoteGame.load(gameid, client=self)
+            self.game = RemoteGame.loadFromDB(gameid, client=self)
             for idx, playerName in enumerate(playerNames.split('//')):
                 self.game.players.byName(playerName).wind = WINDS[idx]
+            if self.isHumanClient():
+                if self.game.handctr != self.table.endValues[0]:
+                    self.game.close()
+                    return 'The data bases for game %1 have different numbers for played hands: Server:%2, Client:%3', \
+                            self.game.seed, self.table.endValues[0], self.game.handctr
+                for player in self.game.players:
+                    if player.balance != self.table.endValues[1][player.wind]:
+                        self.game.close()
+                        return 'The data bases for game %1 have different balances for wind %2: Server:%3, Client:%4', \
+                                self.game.seed, player.wind, self.table.endValues[1][player.wind], player.balance
         else:
             self.game = RemoteGame(playerNames.split('//'), self.table.ruleset,
                 shouldSave=shouldSave, gameid=gameid, seed=seed, client=self,
@@ -207,10 +218,11 @@ class Client(pb.Referenceable):
         as possible with limited computing resources, it stands on
         no theoretical basis"""
         hand = self.game.myself.computeHandContent()
-        hiddenTiles = sum((x.pairs.lower() for x in hand.hiddenMelds), [])
-        tiles = list(sorted(set(hiddenTiles), key=elementKey))
-        candidates = list(TileAI(x) for x in tiles)
         groupCounts = IntDict() # counts for tile groups (sbcdw), exposed and concealed
+        hiddenTiles = sum((x.pairs.lower() for x in hand.hiddenMelds), [])
+        for tile in hiddenTiles:
+            groupCounts[tile[0]] += 1
+        candidates = list(TileAI(x) for x in sorted(set(hiddenTiles), key=elementKey))
         declaredGroupCounts = IntDict()
         for tile in sum((x.pairs.lower() for x in hand.declaredMelds), []):
             groupCounts[tile[0]] += 1
@@ -218,7 +230,6 @@ class Client(pb.Referenceable):
         for candidate in candidates:
             preference = candidate.preference
             group, value = candidate.name
-            groupCounts[group] += 1
             candidate.occurrence = hiddenTiles.count(candidate.name)
             candidate.dangerous = candidate.name in self.game.dangerousTiles
             if candidate.dangerous:
@@ -227,7 +238,7 @@ class Client(pb.Referenceable):
                 preference += 10
             elif candidate.occurrence == 2:
                 preference += 5
-            preference += Client.groupPrefs[group]
+            preference += self.groupPrefs[group]
             if value in '19':
                 preference += 2
             if self.game.visibleTiles[candidate.name] == 3:
@@ -240,14 +251,16 @@ class Client(pb.Referenceable):
             group = candidate.name[0]
             groupCount = groupCounts[group]
             if group in 'sbc':
+                # count tiles with a different color:
                 if groupCount == 1:
                     candidate.preference -= 2
-                elif groupCount > 8:
-                    # do not go for color game if we already declared something in another color:
-                    if not any(declaredGroupCounts[x] for x in 'sbc' if x != group):
-                        # improvement: less if we already have a concealed pung of another color
-                        # improvement: pref for pong only / 0value game
-                        candidate.preference += (groupCount-5) * 2
+                else:
+                    otherGC = sum(groupCounts[x] for x in 'sbc' if x != group)
+                    if otherGC:
+                        if groupCount > 8 or otherGC < 5:
+                            # do not go for color game if we already declared something in another color:
+                            if not any(declaredGroupCounts[x] for x in 'sbc' if x != group):
+                                candidate.preference += 20 // otherGC
             elif group == 'w' and groupCount > 8:
                 candidate.preference += 10
             elif group == 'd' and groupCount > 7:
@@ -255,7 +268,7 @@ class Client(pb.Referenceable):
         self.weighCallingHand(hand, candidates)
         candidates = sorted(candidates, key=lambda x: x.preference)
         if Debug.robotAI:
-            logDebug('%s: %s' % (self.game.myself ,  ' '.join(str(x) for x in candidates)))
+            logDebug('%s: %s' % (self.game.myself, ' '.join(str(x) for x in candidates)))
         # return tile with lowest preference:
         return candidates[0].name.capitalize()
 
@@ -471,4 +484,4 @@ class TileAI(object):
 
 class Client1(Client):
     """alternative AI class"""
-    pass
\ No newline at end of file
+    pass
