@@ -1,6 +1,6 @@
 # -*- coding: utf-8 -*-
 
-"""Copyright (C) 2009,2010 Wolfgang Rohdewald <wolfgang@rohdewald.de>
+"""Copyright (C) 2009-2011 Wolfgang Rohdewald <wolfgang@rohdewald.de>
 
 kajongg is free software you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
@@ -14,7 +14,7 @@ GNU General Public License for more details.
 
 You should have received a copy of the GNU General Public License
 along with this program if not, write to the Free Software
-Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
+Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
 
 
 
@@ -69,6 +69,7 @@ def stateName(state):
 def elementKey(element):
     """to be used in sort() and sorted() as key=. Sort by tile type, value, case.
     element can also be a meld from the summary."""
+    assert len(element) >= 2, 'elementKey wrong:%s' % element
     group = element[0].lower()
     aPos = chr('xdwsbcfy'.index(group) + ord('0'))
     bPos = element[1].lower()
@@ -116,6 +117,7 @@ class Score(object):
         self.points = type(self.points)(points)
         self.doubles = type(self.doubles)(doubles)
         self.limits = type(self.limits)(limits)
+        self.__lastUnit = None # the last unit which had a value
 
     unitNames = [m18nE('points'), m18nE('doubles'), m18nE('limits')]
 
@@ -164,6 +166,8 @@ class Score(object):
                 return 1
             elif self.limits:
                 return 2
+            elif self.__lastUnit:                 # pylint: disable=W0212
+                return self.__lastUnit         # pylint: disable=W0212
             else:
                 return 0
         def fset(self, unit):
@@ -171,6 +175,7 @@ class Score(object):
             oldValue = self.value
             self.clear()
             self.__setattr__(english(Score.unitName(unit)), oldValue)
+            self.__lastUnit = unit
         return property(**locals())
 
     @apply
@@ -224,7 +229,9 @@ class Score(object):
         if limitPoints is None:
             limitPoints = self.limitPoints
         if limitPoints is None:
-            raise Exception('Score.total: limitPoints unknown')
+            if self.limits or self.points:
+                raise Exception('Score.total: limitPoints unknown for %s' % self)
+            return 0
         if self.limits:
             return int(round(self.limits * limitPoints))
         else:
@@ -324,7 +331,7 @@ class Meld(object):
     identical with the name of the corresponding real tile while tiles
     are added or removed. See end of SelectorBoard.meldVariants()."""
 
-    tileNames = {'x':m18nc('kajongg','hidden'), 's': m18nc('kajongg','stone') ,
+    tileNames = {'x':m18nc('kajongg','hidden'), 's': m18nc('kajongg','stone'),
         'b': m18nc('kajongg','bamboo'), 'c':m18nc('kajongg','character'),
         'w':m18nc('kajongg','wind'), 'd':m18nc('kajongg','dragon'),
         'f':m18nc('kajongg','flower'), 'y':m18nc('kajongg','season')}
@@ -373,7 +380,7 @@ class Meld(object):
         fStr = m18nc('kajongg', '%1 doubles', self.score.doubles) if self.score.doubles else ''
         score = ' '.join([pStr, fStr])
         return u'%s %s %s %s:   %s' % (stateName(self.state),
-            meldName(self.meldType) , which, value, score)
+            meldName(self.meldType), which, value, score)
 
     def __getitem__(self, index):
         """Meld[x] returns Tile # x """
