@@ -2,7 +2,7 @@
 # -*- coding: utf-8 -*-
 
 """
-Copyright (C) 2009,2010 Wolfgang Rohdewald <wolfgang@rohdewald.de>
+Copyright (C) 2009-2011 Wolfgang Rohdewald <wolfgang@rohdewald.de>
 
 kajongg is free software you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
@@ -16,7 +16,7 @@ GNU General Public License for more details.
 
 You should have received a copy of the GNU General Public License
 along with this program if not, write to the Free Software
-Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
+Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
 
 The DBPasswordChecker is based on an example from the book
 Twisted Network Programming Essentials by Abe Fettig. Copyright 2006
@@ -30,14 +30,17 @@ InternalParameters.isServer = True
 from util import initLog
 initLog('kajonggserver')
 
+from PyQt4.QtCore import QCoreApplication
 from twisted.spread import pb
 from twisted.internet import error
 from twisted.internet.defer import maybeDeferred, fail
 from twisted.internet.address import UNIXAddress
 from zope.interface import implements
 from twisted.cred import checkers, portal, credentials, error as credError
+from twisted.internet import reactor
 
-from game import RemoteGame, Players
+from game import RemoteGame
+from player import Players
 from wall import WallEmpty
 from client import Client
 from query import Transaction, Query, initDb
@@ -144,7 +147,7 @@ class Table(object):
         return property(**locals())
 
     def msg(self):
-        """return a tuple with the attributes to be sent to the client"""
+        """return the table attributes to be sent to the client"""
         game = self.game or self.preparedGame
         onlineNames = [x.name for x in self.users]
         if game:
@@ -152,8 +155,12 @@ class Table(object):
         else:
             names = tuple(x.name for x in self.users)
         online = tuple(bool(x in onlineNames) for x in names)
-        return tuple([self.tableid, game.gameid if game else None, self.status, self.ruleset.toList(),
-                self.playOpen, self.autoPlay, self.seed,  names, online])
+        if game:
+            endValues = game.handctr, dict((x.wind, x.balance) for x in game.players)
+        else:
+            endValues = None
+        return self.tableid, game.gameid if game else None, self.status, self.ruleset.toList(), \
+                self.playOpen, self.autoPlay, self.seed, names, online, endValues
 
     def maxSeats(self):
         """for a new game: 4. For a suspended game: The
@@ -303,12 +310,17 @@ class Table(object):
 
     def startGame(self, requests):
         """if all players said ready, start the game"""
+        mayStart = True
         for msg in requests:
-            if msg.answer == Message.NO:
+            if msg.answer == Message.NO or len(requests) < 4:
                 # this player answered "I am not ready", exclude her from table
+                # a player might already have logged of from the table. So if we
+                # are not 4 anymore, all players must leave the table
                 self.server.leaveTable(msg.player.remote, self.tableid)
                 self.preparedGame = None
-                return
+                mayStart = False
+        if not mayStart:
+            return
         self.game = self.preparedGame
         elementIter = iter(elements.all(self.game.ruleset))
         for tile in self.game.wall.tiles:
@@ -465,6 +477,8 @@ class Table(object):
 
     def endHand(self, dummyResults=None):
         """hand is over, show all concealed tiles to all players"""
+        if not self.game:
+            return
         if self.game.playOpen:
             self.saveHand()
         else:
@@ -653,7 +667,7 @@ class Table(object):
         if not answers:
             return
         for answer in answers:
-            msg = '%s <- %s' % (self.tableid, unicode(answer))
+            msg = '%s <-  %s' % (self.tableid, unicode(answer))
             if InternalParameters.showTraffic:
                 logDebug(msg)
             with Duration(msg):
@@ -666,7 +680,11 @@ class Table(object):
         if not answers:
             self.nextTurn()
 
-    def tellAll(self, player, command, callback=None,  **kwargs):
+    def notMoved(self, requests):
+        """a player sent a notification, has already been processed"""
+        self.processAnswers(requests) # we still want debugging output
+
+    def tellAll(self, player, command, callback=None, **kwargs):
         """tell something to all players"""
         block = DeferredBlock(self)
         block.tellAll(player, command, **kwargs)
@@ -810,6 +828,9 @@ class MJServer(object):
                         else:
                             self.leaveTable(user, table.tableid)
                 self.users.remove(user)
+        if InternalParameters.socket and not self.users and reactor.running:
+            logInfo('local server terminates. Reason: last client disconnected')
+            reactor.stop()
 
     def loadSuspendedTables(self, user):
         """loads all yet unloaded suspended tables where this
@@ -840,7 +861,7 @@ class MJServer(object):
                     table = Table(self, None, Ruleset.cached(ruleset, used=True), playOpen, autoPlay=False, seed=seed)
                     table.tableid = 1000 + gameid
                     table.status = m18ncE('table status', 'Suspended') + suspendTime
-                    table.preparedGame = RemoteGame.load(gameid, None, cacheRuleset=True)
+                    table.preparedGame = RemoteGame.loadFromDB(gameid, None, cacheRuleset=True)
                     self.suspendedTables[gameid] = table
 
 class User(pb.Avatar):
@@ -885,7 +906,7 @@ class User(pb.Avatar):
         """perspective_* methods are to be called remotely"""
         self.detached(None)
     def __str__(self):
-        return '%d:%s' % (id(self) % 10000,  self.name)
+        return '%d:%s' % (id(self) % 10000, self.name)
 
 class MJRealm(object):
     """connects mind and server"""
@@ -897,15 +918,17 @@ class MJRealm(object):
     def requestAvatar(self, avatarId, mind, *interfaces):
         """as the tutorials do..."""
         if not pb.IPerspective in interfaces:
-            raise NotImplementedError,  "No supported avatar interface"
+            raise NotImplementedError, "No supported avatar interface"
         avatar = User(avatarId)
         avatar.server = self.server
         avatar.attached(mind)
         return pb.IPerspective, avatar, lambda a = avatar:a.detached(mind)
 
+# pylint: disable=W0404
+# pylint does not like imports within functions
+
 def kajonggServer():
     """start the server"""
-    from twisted.internet import reactor
     from optparse import OptionParser
     parser = OptionParser()
     parser.add_option('', '--port', dest='port', help=m18n('the server will listen on PORT'),
@@ -915,8 +938,8 @@ def kajonggServer():
     parser.add_option('', '--showsql', dest='showsql', action='store_true',
         help=m18n('show database SQL commands'), default=False)
     parser.add_option('', '--db', dest='dbpath', help=m18n('name of the database'), default=None)
-    parser.add_option('', '--socket', dest='socket', action='store_true',
-        help=m18n('listen on UNIX socket %1', socketName()), default=False)
+    parser.add_option('', '--local', dest='socket', action='store_true',
+        help=m18n('start a local game server', socketName()), default=False)
     (options, args) = parser.parse_args()
     if args and ''.join(args):
         logWarning(m18n('unrecognized arguments:%1', ' '.join(args)))
@@ -937,8 +960,14 @@ def kajonggServer():
     loadPredefinedRulesets()
     try:
         if InternalParameters.socket:
-            reactor.listenUNIX(InternalParameters.socket, pb.PBServerFactory(kajonggPortal))
+            if os.name == 'nt':
+                logInfo('local server listening on 127.0.0.1 port %d' % port)
+                reactor.listenTCP(port, pb.PBServerFactory(kajonggPortal), interface='127.0.0.1')
+            else:
+                logInfo('local server listening on UNIX socket %s' % InternalParameters.socket)
+                reactor.listenUNIX(InternalParameters.socket, pb.PBServerFactory(kajonggPortal))
         else:
+            logInfo('server listening on port %d' % port)
             reactor.listenTCP(port, pb.PBServerFactory(kajonggPortal))
     except error.CannotListenError, errObj:
         logWarning(errObj)
@@ -953,3 +982,6 @@ def profileMe():
     statistics = pstats.Stats('prof')
     statistics.sort_stats('cumulative')
     statistics.print_stats(40)
+
+# we need this so we can load SQL driver plugins on Windows
+SERVERAPP = QCoreApplication([])
