@@ -23,8 +23,6 @@
 #include <QByteArray>
 #include <QString>
 
-// #define ENABLE_SOUND_SUPPORT // Ian W. disabled sound - 19 Dec 2010.
-
 enum Owner {SYSTEM, USER};
 
 const char FREE      = ' ';
@@ -162,7 +160,8 @@ enum Setting       {PLAY_SOUNDS,			// Sound effects on/off.
                     CLICK_KEY, HOLD_KEY, 		// Key-control method.
                     NORMAL_SPEED, BEGINNER_SPEED,	// Preset game-speeds.
                     CHAMPION_SPEED,
-                    INC_SPEED, DEC_SPEED};		// Adjustments of speed.
+                    INC_SPEED, DEC_SPEED,		// Adjustments of speed.
+                    PLAY_STEPS};			// Footsteps on/off.
 
 const int  ConcreteWall = 1;
 
