@@ -1,5 +1,5 @@
 /*
- *  Copyright 2006-2009  Parker Coates <parker.coates@gmail.com>
+ *  Copyright 2006-2009  Parker Coates <parker.coates@kdemail.net>
  *
  *  This file is part of Killbots.
  *
@@ -19,15 +19,16 @@
 
 #include "sprite.h"
 
-#include "render.h"
+#include "renderer.h"
 
 #include <QtGui/QPainter>
 
 
 Killbots::Sprite::Sprite()
-  : QGraphicsPixmapItem()
+  : KGameRenderedItem( Renderer::self(), QString() )
 {
 	setShapeMode( QGraphicsPixmapItem::BoundingRectShape );
+	setTransformationMode( Qt::FastTransformation );
 }
 
 
@@ -44,7 +45,27 @@ Killbots::SpriteType Killbots::Sprite::spriteType() const
 
 void Killbots::Sprite::setSpriteType( Killbots::SpriteType type )
 {
+	Q_ASSERT( type != NoSprite );
+
 	m_type = type;
+
+	switch ( m_type )
+	{
+	case Hero :
+		setSpriteKey( "hero" );
+		break;
+	case Robot :
+		setSpriteKey( "enemy" );
+		break;
+	case Fastbot :
+		setSpriteKey( "fastenemy" );
+		break;
+	case Junkheap :
+		setSpriteKey( "junkheap" );
+		break;
+	default :
+		break;
+	}
 }
 
 
@@ -80,23 +101,15 @@ void Killbots::Sprite::advanceGridPosQueue()
 }
 
 
-void Killbots::Sprite::setSize( QSize size )
-{
-	static QHash<SpriteType, QString> elementHash;
-	if ( elementHash.isEmpty() )
-	{
-		elementHash.insert( Hero, "hero" );
-		elementHash.insert( Robot, "enemy" );
-		elementHash.insert( Fastbot, "fastenemy" );
-		elementHash.insert( Junkheap, "junkheap" );
-	}
-
-	setOffset( -0.5 * QPointF( size.width(), size.height() ) );
-	setPixmap( Render::renderElement( elementHash.value( m_type ), size ) );
-}
-
-
 int Killbots::Sprite::type() const
 {
 	return Type;
 }
+
+
+void Killbots::Sprite::receivePixmap( const QPixmap & pixmap )
+{
+	KGameRenderedItem::receivePixmap( pixmap );
+	setOffset( -0.5 * QPointF( pixmap.width(), pixmap.height() ) );
+}
+
