@@ -48,30 +48,30 @@ class Spider : public DealerScene
 
 public:
     virtual void initialize();
-    virtual void moveCardsToPile( QList<KCard*> cards, KCardPile * pile, int duration );
     virtual void mapOldId(int id);
     virtual int oldId() const;
     virtual QList<QAction*> configActions() const;
 
 protected:
-    virtual QString getGameState();
-    virtual void setGameState(const QString &stream);
+    virtual QString getGameState() const;
+    virtual void setGameState( const QString & state );
     virtual QString getGameOptions() const;
-    virtual void setGameOptions(const QString &options);
+    virtual void setGameOptions( const QString & options );
     virtual bool checkAdd(const PatPile * pile, const QList<KCard*> & oldCards, const QList<KCard*> & newCards) const;
     virtual bool checkRemove(const PatPile * pile, const QList<KCard*> & cards) const;
+    virtual void cardsMoved( const QList<KCard*> & cards, KCardPile * oldPile, KCardPile * newPile );
     virtual void restart( const QList<KCard*> & cards );
 
 protected slots:
     virtual bool newCards();
+    virtual void animationDone();
 
 private slots:
     void gameTypeChanged();
-    void checkAllForRuns();
 
 private:
-    bool checkPileDeck( KCardPile * pile, bool checkForDemo = true );
-    QList<KCard*> getRun(KCard *c) const;
+    bool pileHasFullRun( KCardPile * pile );
+    void moveFullRunToLeg( KCardPile * pile );
     void setSuits(int s);
     void createDeck();
     QPointF randomPos();
@@ -82,6 +82,7 @@ private:
     PatPile *redeals[5];
     int m_redeal;
     int m_suits;
+    QList<KCardPile*> m_pilesWithRuns;
 
     KSelectAction *options;
 
