@@ -3,7 +3,7 @@
  *   Copyright 1997   Mario Weilguni <mweilguni@sime.com>                  *
  *   Copyright 2002-2004  Dave Corrie <kde@davecorrie.com>                 *
  *   Copyright 2007  Mauricio Piacentini <mauricio@tabuleiro.com>          *
- *   Copyright 2009,2010  Frederik Schwarzer <schwarzerf@gmail.com>        *
+ *   Copyright 2009-2011  Frederik Schwarzer <schwarzerf@gmail.com>        *
  *                                                                         *
  *   This program is free software; you can redistribute it and/or modify  *
  *   it under the terms of the GNU General Public License as published by  *
@@ -69,15 +69,15 @@ bool PossibleMove::isInPath(int x, int y) const
 }
 
 Board::Board(QWidget *parent)
-  : QWidget(parent),
-    m_markX(0), m_markY(0),
-    m_field(0),
-    m_xTiles(0), m_yTiles(0),
-    m_delay(0), m_level(0), m_shuffle(0),
-    m_isPaused(false), m_isStuck(false), m_isOver(false), m_cheat(false),
-    m_gravityFlag(true), m_solvableFlag(false), m_chineseStyleFlag(false), m_tilesCanSlideFlag(false),
-    m_highlightedTile(-1), m_connectionTimeout(0),
-    m_paintConnection(false), m_paintPossibleMoves(false), m_paintInProgress(false), m_media(0)
+    : QWidget(parent),
+      m_markX(0), m_markY(0),
+      m_field(0),
+      m_xTiles(0), m_yTiles(0),
+      m_delay(0), m_level(0), m_shuffle(0),
+      m_gameState(Normal), m_cheat(false),
+      m_gravityFlag(true), m_solvableFlag(false), m_chineseStyleFlag(false), m_tilesCanSlideFlag(false),
+      m_highlightedTile(-1), m_connectionTimeout(0),
+      m_paintConnection(false), m_paintPossibleMoves(false), m_paintInProgress(false), m_media(0)
 {
     m_tileRemove1.first = -1;
 
@@ -128,7 +128,7 @@ void Board::loadSettings()
     setDelay(s_delay[Prefs::speed()]);
     setSoundsEnabled(Prefs::sounds());
 
-    if( m_level != Prefs::level() ) {
+    if (m_level != Prefs::level()) {
         newGame();
     }
     m_level = Prefs::level();
@@ -192,7 +192,7 @@ void Board::setField(int x, int y, int value)
 {
     if (x < 0 || y < 0 || x >= xTiles() || y >= yTiles()) {
         kFatal() << "Attempted write to invalid field position "
-        "(" << x << "," << y << ")";
+                 "(" << x << "," << y << ")";
     }
 
     m_field[y * xTiles() + x] = value;
@@ -204,7 +204,7 @@ int Board::field(int x, int y) const
 #ifdef DEBUGGING
     if (x < -1 || y < -1 || x > xTiles() || y > yTiles()) {
         kFatal() << "Attempted read from invalid field position "
-        "(" << x << "," << y << ")";
+                 "(" << x << "," << y << ")";
     }
 #endif
 
@@ -263,7 +263,8 @@ bool Board::gravity(int column, bool update)
     return isAffected;
 }
 
-void Board::unmarkTile() {
+void Board::unmarkTile()
+{
     // if nothing is marked, nothing to do
     if (m_markX == -1 || m_markY == -1) {
         return;
@@ -289,13 +290,15 @@ void Board::mousePressEvent(QMouseEvent *e)
     if (m_paintInProgress) {
         return;
     }
-    if (m_isOver) {
-        newGame();
-        return;
-    }
-    if (m_isPaused) {
-        setPauseEnabled(false);
-        return;
+    switch (m_gameState) {
+        case Over:
+            newGame();
+            return;
+        case Paused:
+            setPauseEnabled(false);
+            return;
+        case Stuck:
+            return;
     }
     // Calculate field position
     int posX = (e->pos().x() - xOffset()) / (m_tiles.qWidth() * 2);
@@ -432,9 +435,7 @@ void Board::resizeBoard()
 
 void Board::newGame()
 {
-    m_isOver = false;
-    m_isPaused = false;
-    m_isStuck = false;
+    m_gameState = Normal;
     setCheatModeEnabled(false);
 
     m_markX = -1;
@@ -638,57 +639,72 @@ void Board::updateField(int x, int y)
     update(r);
 }
 
+void Board::showInfoRect(QPainter &p, const QString &message)
+{
+    int boxWidth = width() * 0.6;
+    int boxHeight = height() * 0.6;
+    QRect contentsRect = QRect((width() - boxWidth) / 2, (height() - boxHeight) / 2, boxWidth, boxHeight);
+    QFont font;
+    int fontsize = boxHeight / 13;
+    font.setPointSize(fontsize);
+    p.setFont(font);
+    p.setBrush(QBrush(QColor(100, 100, 100, 150)));
+    p.setRenderHint(QPainter::Antialiasing);
+    p.drawRoundedRect(contentsRect, 10, 10);
+
+    p.drawText(contentsRect, Qt::AlignCenter | Qt::TextWordWrap, message);
+}
+
+void Board::drawTiles(QPainter &p, QPaintEvent *e)
+{
+    int w = m_tiles.width();
+    int h = m_tiles.height();
+    int fw = m_tiles.qWidth() * 2;
+    int fh = m_tiles.qHeight() * 2;
+    for (int i = 0; i < xTiles(); ++i) {
+        for (int j = 0; j < yTiles(); ++j) {
+            int tile = field(i, j);
+            if (tile == EMPTY) {
+                continue;
+            }
+
+            int xpos = xOffset() + i * fw;
+            int ypos = yOffset() + j * fh;
+            QRect r(xpos, ypos, w, h);
+            if (e->rect().intersects(r)) {
+                if (isTileHighlighted(i, j)) {
+                    p.drawPixmap(xpos, ypos, m_tiles.selectedTile(1));
+                } else {
+                    p.drawPixmap(xpos, ypos, m_tiles.unselectedTile(1));
+                }
+
+                //draw face
+                p.drawPixmap(xpos, ypos, m_tiles.tileface(tile - 1));
+            }
+        }
+    }
+}
+
 void Board::paintEvent(QPaintEvent *e)
 {
-    QRect ur = e->rect();            // rectangle to update
+    QRect ur = e->rect(); // rectangle to update
     QPainter p(this);
     p.fillRect(ur, m_background.getBackground());
 
-    if (m_isPaused || m_isOver) {
-        int boxWidth = width() * 0.7;
-        int boxHeight = height() * 0.8;
-        QRect contentsRect = QRect((width() - boxWidth) / 2, (height() - boxHeight) / 2, boxWidth, boxHeight);
-        QFont font;
-        int fontsize = 36;
-        font.setPointSize(fontsize);
-        p.setFont(font);
-        //     p.setPen(QPen(QColor("black")));
-        p.setBrush(QBrush(QColor(100, 100, 100, 150)));
-        p.setRenderHint(QPainter::Antialiasing);
-        p.drawRoundedRect(contentsRect, 10, 10);
-
-        if (m_isPaused) {
-            p.drawText(contentsRect, Qt::AlignCenter|Qt::TextWordWrap, i18n("Game Paused\nClick to resume game."));
-        } else if (m_isOver) {
-            p.drawText(contentsRect, Qt::AlignCenter|Qt::TextWordWrap, i18n("Game Over\nClick to start a new game."));
-        }
-    } else {
-        int w = m_tiles.width();
-        int h = m_tiles.height();
-        int fw = m_tiles.qWidth() * 2;
-        int fh = m_tiles.qHeight() * 2;
-        for (int i = 0; i < xTiles(); ++i) {
-            for (int j = 0; j < yTiles(); ++j) {
-                int tile = field(i, j);
-                if (tile == EMPTY) {
-                    continue;
-                }
-
-                int xpos = xOffset() + i * fw;
-                int ypos = yOffset() + j * fh;
-                QRect r(xpos, ypos, w, h);
-                if (e->rect().intersects(r)) {
-                    if (isTileHighlighted(i, j)) {
-                        p.drawPixmap(xpos, ypos, m_tiles.selectedTile(1));
-                    } else {
-                        p.drawPixmap(xpos, ypos, m_tiles.unselectedTile(1));
-                    }
-
-                    //draw face
-                    p.drawPixmap(xpos, ypos, m_tiles.tileface(tile - 1));
-                }
-            }
-        }
+    switch (m_gameState) {
+        case Normal:
+            drawTiles(p, e);
+            break;
+        case Paused:
+            showInfoRect(p, i18n("Game Paused\nClick to resume game."));
+            break;
+        case Stuck:
+            drawTiles(p, e);
+            showInfoRect(p, i18n("Game Stuck\nNo more moves possible."));
+            break;
+        case Over:
+            showInfoRect(p, i18n("Game Over\nClick to start a new game."));
+            break;
     }
 
     if (m_paintConnection) {
@@ -1818,7 +1834,7 @@ bool Board::hint_I(PossibleMoves &possibleMoves) const
         for (int y = 0; y < yTiles(); ++y) {
             int tile = field(x, y);
             if (tile != EMPTY && done[tile - 1] != 4) {
-                // for all these types of tile search path's
+                // for all these types of tile search paths
                 for (int xx = 0; xx < xTiles(); ++xx) {
                     for (int yy = 0; yy < yTiles(); ++yy) {
                         if (xx != x || yy != y) {
@@ -1875,9 +1891,9 @@ bool Board::solvable(bool noRestore)
     PossibleMoves p;
     while (hint_I(p)) {
         kFatal(!tilesMatch(field(p.first().m_path.first().x, p.first().m_path.first().y), field(p.first().m_path.last().x, p.first().m_path.last().y)))
-        << "Removing unmatched tiles: (" << p.first().m_path.first().x << "," << p.first().m_path.first().y << ") => "
-        << field(p.first().m_path.first().x, p.first().m_path.first().y) << " (" << p.first().m_path.last().x << "," << p.first().m_path.last().y << ") => "
-        << field(p.first().m_path.last().x, p.first().m_path.last().y);
+                << "Removing unmatched tiles: (" << p.first().m_path.first().x << "," << p.first().m_path.first().y << ") => "
+                << field(p.first().m_path.first().x, p.first().m_path.first().y) << " (" << p.first().m_path.last().x << "," << p.first().m_path.last().y << ") => "
+                << field(p.first().m_path.last().x, p.first().m_path.last().y);
         setField(p.first().m_path.first().x, p.first().m_path.first().y, EMPTY);
         setField(p.first().m_path.last().x, p.first().m_path.last().y, EMPTY);
     }
@@ -1955,13 +1971,14 @@ void Board::setTilesCanSlideFlag(bool enabled)
 
 void Board::setPauseEnabled(bool enabled)
 {
-    if (m_isPaused == enabled) {
+    if ((m_gameState == Paused && enabled) || m_gameState == Stuck) {
         return;
     }
-    m_isPaused = enabled;
-    if (m_isPaused) {
+    if (enabled) {
+        m_gameState = Paused;
         m_gameClock.pause();
     } else {
+        m_gameState = Normal;
         m_gameClock.resume();
     }
     emit changed();
@@ -1974,7 +1991,7 @@ QSize Board::sizeHint() const
     if (dpi < 75) {
         dpi = 75;
     }
-    return QSize(9*dpi, 7*dpi);
+    return QSize(9 * dpi, 7 * dpi);
 }
 
 void Board::resetTimer()
@@ -2002,13 +2019,14 @@ void Board::resetRedo()
 
 void Board::setGameStuckEnabled(bool enabled)
 {
-    if (m_isStuck == enabled) {
+    if (m_gameState == Stuck && enabled) {
         return;
     }
-    m_isStuck = enabled;
-    if (m_isStuck) {
+    if (enabled) {
+        m_gameState = Stuck;
         m_gameClock.pause();
     } else {
+        m_gameState = Normal;
         m_gameClock.resume();
     }
     emit changed();
@@ -2017,10 +2035,10 @@ void Board::setGameStuckEnabled(bool enabled)
 
 void Board::setGameOverEnabled(bool enabled)
 {
-    if (m_isOver == enabled) {
+    if (m_gameState == Over && enabled) {
         return;
     }
-    m_isOver = enabled;
+    m_gameState = Over;
     emit changed();
     update();
 }
@@ -2036,17 +2054,17 @@ void Board::setCheatModeEnabled(bool enabled)
 
 bool Board::isOver() const
 {
-    return m_isOver;
+    return m_gameState == Over;
 }
 
 bool Board::isPaused() const
 {
-    return m_isPaused;
+    return m_gameState == Paused;
 }
 
 bool Board::isStuck() const
 {
-    return m_isStuck;
+    return m_gameState == Stuck;
 }
 
 bool Board::hasCheated() const
