@@ -1054,7 +1054,7 @@ void KGameWindow::slotGroupChatJoined (const XMPP::Jid & jid)
   /*    <message type="chat" to="kleag@localhost" id="aabca" >
   <body>hello</body>
   </message>*/
-  XMPP::Message message(m_groupchatRoom+'@'+m_groupchatHost);
+  XMPP::Message message(QString(m_groupchatRoom+'@'+m_groupchatHost));
   message.setType("groupchat");
   message.setId(QUuid::createUuid().toString().remove('{').remove('}').remove('-'));
   message.setBody("Hello, I'm a KsirK Game");
