@@ -65,6 +65,7 @@
 #include "settings.h"
 #include "config.h"
 
+using namespace ksudoku;
 
 void KSudoku::onCompleted(bool isCorrect, const QTime& required, bool withHelp) {
 	if(!isCorrect) {
@@ -73,37 +74,47 @@ void KSudoku::onCompleted(bool isCorrect, const QTime& required, bool withHelp)
 	}
 
 	QString msg;
- 	int secs = QTime(0,0).secsTo(required);
- 	int mins = secs / 60;
- 	secs = secs % 60;
+	int secs = QTime(0,0).secsTo(required);
+	int mins = secs / 60;
+	secs = secs % 60;
 
 	if(withHelp)
-		msg = i18n("Congratulations! You made it in %1 minutes and %2 seconds. With some tricks.", mins, secs);
+		if (mins == 0)
+			msg = i18np("Congratulations! You made it in 1 second. With some tricks.", "Congratulations! You made it in %1 seconds. With some tricks.", secs);
+		else if (secs == 0)
+			msg = i18np("Congratulations! You made it in 1 minute. With some tricks.", "Congratulations! You made it in %1 minutes. With some tricks.", mins);
+		else
+			msg = i18nc("The two parameters are strings like '2 minutes' or '1 second'.", "Congratulations! You made it in %1 and %2. With some tricks.", i18np("1 minute", "%1 minutes", mins), i18np("1 second", "%1 seconds", secs));
 	else
-		msg = i18n("Congratulations! You made it in %1 minutes and %2 seconds.", mins, secs);
+		if (mins == 0)
+			msg = i18np("Congratulations! You made it in 1 second.", "Congratulations! You made it in %1 seconds.", secs);
+		else if (secs == 0)
+			msg = i18np("Congratulations! You made it in 1 minute.", "Congratulations! You made it in %1 minutes.", mins);
+		else
+			msg = i18nc("The two parameters are strings like '2 minutes' or '1 second'.", "Congratulations! You made it in %1 and %2.", i18np("1 minute", "%1 minutes", mins), i18np("1 second", "%1 seconds", secs));
 
 	KMessageBox::information(this, msg);
 
 }
 
-void KSudoku::updateStatusBar()
-{
-	QString m="";
-// 	QWidget* current = m_tabs->currentPage();
-// 	if(KsView* view = dynamic_cast<KsView*>(current))
-// 		m = view->status();
-// 	if(currentView())
-// 		m = currentView()->status();
-
-	// TODO fix this: add new status bar generation code
-
-	statusBar()->showMessage(m);
-}
+// void KSudoku::updateStatusBar()
+// {
+// 	QString m="";
+// // 	QWidget* current = m_tabs->currentPage();
+// // 	if(KsView* view = dynamic_cast<KsView*>(current))
+// // 		m = view->status();
+// // 	if(currentView())
+// // 		m = currentView()->status();
+// 
+// 	// TODO fix this: add new status bar generation code
+// 
+// 	statusBar()->showMessage(m);
+// }
 
 KSudoku::KSudoku()
 	: KXmlGuiWindow(), m_gameVariants(new GameVariantCollection(this, true))
 {
-	setObjectName("ksudoku");
+	setObjectName( QLatin1String("ksudoku" ));
 
 	m_gameWidget = 0;
 	m_gameUI = 0;
@@ -113,9 +124,7 @@ KSudoku::KSudoku()
 	// then, setup our actions
 	setupActions();
 
-	// and a status bar
-	statusBar()->show();
-	setupGUI();
+	setupGUI(ToolBar | Keys | Save | Create);
 
 	wrapper = new QWidget();
 	(void) new QHBoxLayout(wrapper);
@@ -137,13 +146,10 @@ KSudoku::KSudoku()
 
 	updateShapesList();
 
-	m_symbols.setEnabledTables(Settings::symbols());
-
-
-	QTimer *timer = new QTimer( this );
-	connect( timer, SIGNAL(timeout()), this, SLOT(updateStatusBar()) );
-	updateStatusBar();
-	timer->start( 1000); //TODO PORT, false ); // 2 seconds single-shot timer
+// 	QTimer *timer = new QTimer( this );
+// 	connect( timer, SIGNAL(timeout()), this, SLOT(updateStatusBar()) );
+// 	updateStatusBar();
+// 	timer->start( 1000); //TODO PORT, false ); // 2 seconds single-shot timer
 }
 
 KSudoku::~KSudoku()
@@ -157,7 +163,7 @@ void KSudoku::updateShapesList()
 	GameVariant* variant = 0;
 
 	variant = new SudokuGame(i18n("Sudoku Standard (9x9)"), 9, m_gameVariants);
-	variant->setDescription(i18n("The classic and fashion game"));
+	variant->setDescription(i18n("The classic and fashionable game"));
 	variant->setIcon("ksudoku-ksudoku_9x9");
 	variant = new SudokuGame(i18n("Sudoku 16x16"), 16, m_gameVariants);
 	variant->setDescription(i18n("Sudoku with 16 symbols"));
@@ -210,7 +216,6 @@ void KSudoku::startGame(const Game& game) {
 
 	view->setValueListWidget(m_valueListWidget);
 	view->createView();
-	view->setSymbolTable(m_symbols.selectTable(view->game().order()));
 
 	connect(view, SIGNAL(valueSelected(int)), m_valueListWidget, SLOT(selectValue(int)));
 	connect(m_valueListWidget, SIGNAL(valueSelected(int)), view, SLOT(selectValue(int)));
@@ -221,6 +226,7 @@ void KSudoku::startGame(const Game& game) {
 
 	wrapper->layout()->addWidget(widget);
 	widget->show();
+	widget->setFocus();
 
 	connect(game.interface(), SIGNAL(completed(bool,const QTime&,bool)), SLOT(onCompleted(bool,const QTime&,bool)));
 	connect(game.interface(), SIGNAL(modified(bool)), SLOT(onModified(bool)));
@@ -232,7 +238,7 @@ void KSudoku::startGame(const Game& game) {
 	policy.setVerticalStretch(1);
 	widget->setSizePolicy(policy);
 
-	m_valueListWidget->setCurrentTable(m_symbols.selectTable(view->game().	order()), view->game().order());
+	m_valueListWidget->setMaxValue(view->game().order());
 	m_valueListWidget->selectValue(1);
 	m_valueListWidget->show();
 }
@@ -294,22 +300,20 @@ void KSudoku::dubPuzzle()
 		return;
 	}
 
-	int forks = 0;
 	ksudoku::Puzzle* puzzle = game.puzzle()->dubPuzzle();
-	int state = puzzle->init(game.allValues(), &forks);
+	int state = puzzle->init(game.allValues());
 
 	if(state <= 0) {
-		KMessageBox::information(this, i18n("Sorry, No solutions have been found."));
+		KMessageBox::information(this, i18n("Sorry, no solutions have been found."));
 		delete puzzle;
 		return;
 	} else if(state == 1) {
-		// TODO Remove the (Forks required: %1) after 4.1 The user couldn't care less about that
-		KMessageBox::information(this, i18n("The Puzzle you entered has only one solution. (Forks required: %1)",forks));
+		KMessageBox::information(this, i18n("The Puzzle you entered has only one solution."));
 	} else {
 		KMessageBox::information(this, i18n("The Puzzle you entered has multiple solutions."));
 	}
 
-	if(KMessageBox::questionYesNo(this, i18n("Do you want to play the puzzle now?")) == 3)
+	if(KMessageBox::questionYesNo(this, i18n("Do you want to play the puzzle now?"), i18n("Play Puzzle"), KGuiItem(i18n("Play")), KStandardGuiItem::cancel() ) == KMessageBox::Yes)
 	{
 		startGame(ksudoku::Game(puzzle));
 	}
@@ -355,17 +359,17 @@ void KSudoku::setupActions()
 	KStandardGameAction::solve(this, SLOT(autoSolve()), actionCollection());
 
 	KAction* a = new KAction(this);
-	actionCollection()->addAction("move_dub_puzzle", a);
+	actionCollection()->addAction( QLatin1String( "move_dub_puzzle" ), a);
 	a->setText(i18n("Check"));
-	a->setIcon(KIcon("games-endturn"));
+	a->setIcon(KIcon( QLatin1String( "games-endturn" )));
 	connect(a, SIGNAL(triggered(bool)), SLOT(dubPuzzle()));
 	addAction(a);
 
 	//WEB
 	a = new KAction(this);
-	actionCollection()->addAction("home_page", a);
+	actionCollection()->addAction( QLatin1String( "home_page" ), a);
 	a->setText(i18n("Home Page"));
-	a->setIcon(KIcon("internet-web-browser"));
+	a->setIcon(KIcon( QLatin1String( "internet-web-browser" )));
 	connect(a, SIGNAL(triggered(bool)), SLOT(homepage()));
 }
 
@@ -376,7 +380,7 @@ void KSudoku::adaptActions2View() {
 	m_gameSaveAs->setEnabled(game.isValid());
 	if(game.isValid()) {
 		action("move_undo")->setEnabled(game.canUndo());
-		action("move_undo")->setEnabled(game.canRedo());
+		action("move_redo")->setEnabled(game.canRedo());
 
 		action("move_hint")      ->setEnabled(   game.puzzle()->hasSolution());
 		action("move_solve")     ->setEnabled(   game.puzzle()->hasSolution());
@@ -474,9 +478,15 @@ void KSudoku::gameNew()
 
 	if(!currentView()) return;
 
-	// TODO onyl show this when there is a game running
-	if(KMessageBox::questionYesNo(this, i18n("Do you really want to end this game in order to start a new one?")) != KMessageBox::Yes)
-		return;
+	// only show question when the current game hasn't been finished until now
+	if(!m_gameUI->game().wasFinished()) {
+		if(KMessageBox::questionYesNo(this,
+	                              i18n("Do you really want to end this game in order to start a new one?"),
+	                              i18nc("window title", "Restart Game"),
+	                              KGuiItem(i18nc("button label", "Restart Game")),
+	                              KStandardGuiItem::cancel() ) != KMessageBox::Yes)
+			return;
+	}
 
 	showWelcomeScreen();
 }
@@ -487,7 +497,7 @@ void KSudoku::gameOpen()
 	// the Open shortcut is pressed (usually CTRL+O) or the Open toolbar
 	// button is clicked
 	// standard filedialog
-	KUrl Url = KFileDialog::getOpenUrl(KUrl(), QString(), this, i18n("Open Location"));
+	KUrl Url = KFileDialog::getOpenUrl(KUrl("kfiledialog:///ksudoku"), QString(), this, i18n("Open Location"));
 
 	if (!Url.isEmpty() && Url.isValid())
 	{
@@ -515,7 +525,7 @@ void KSudoku::gameSave()
 	Game game = currentGame();
 	if(!game.isValid()) return;
 
-	if(game.getUrl().isEmpty()) game.setUrl(KFileDialog::getSaveUrl());
+	if(game.getUrl().isEmpty()) game.setUrl(KFileDialog::getSaveUrl(KUrl("kfiledialog:///ksudoku")));
  	if (!game.getUrl().isEmpty() && game.getUrl().isValid())
 		ksudoku::Serializer::store(game, game.getUrl(), this);
 }
@@ -526,7 +536,7 @@ void KSudoku::gameSaveAs()
 	Game game = currentGame();
 	if(!game.isValid()) return;
 
-	game.setUrl(KFileDialog::getSaveUrl());
+	game.setUrl(KFileDialog::getSaveUrl(KUrl("kfiledialog:///ksudoku")));
     if (!game.getUrl().isEmpty() && game.getUrl().isValid())
     	gameSave();
 }
@@ -570,24 +580,18 @@ void KSudoku::optionsPreferences()
 	dialog->addPage(gameConfig, i18nc("Game Section in Config", "Game"), "games-config-options");
 	dialog->addPage(new KGameThemeSelector(dialog, Settings::self(), KGameThemeSelector::NewStuffDisableDownload), i18n("Theme"), "games-config-theme");
 
-	SymbolConfig* symbolConfig = new SymbolConfig(&m_symbols);
-	dialog->addPage(symbolConfig, i18n("Symbol Themes"), "games-config-theme");
-        dialog->setHelp(QString(),"ksudoku");
+    dialog->setHelp(QString(),"ksudoku");
 	connect(dialog, SIGNAL(settingsChanged(const QString&)), SLOT(updateSettings()));
 	dialog->show();
 }
 
 void KSudoku::updateSettings() {
-	m_symbols.setEnabledTables(Settings::symbols());
-
 	Renderer::instance()->loadTheme(Settings::theme());
 
 	KsView* view = currentView();
 	if(view) {
 		int order = view->game().order();
-		SymbolTable* table = m_symbols.selectTable(order);
-		view->setSymbolTable(table);
-		m_valueListWidget->setCurrentTable(table, order);
+		m_valueListWidget->setMaxValue(order);
 
 		view->settingsChanged();
 	}
@@ -595,11 +599,11 @@ void KSudoku::updateSettings() {
 	emit settingsChanged();
 }
 
-void KSudoku::changeStatusbar(const QString& text)
-{
-    // display the text on the statusbar
-    statusBar()->showMessage(text);
-}
+// void KSudoku::changeStatusbar(const QString& text)
+// {
+//     // display the text on the statusbar
+//     statusBar()->showMessage(text);
+// }
 
 void KSudoku::changeCaption(const QString& text)
 {
